@@ -1,14 +1,15 @@
 #!/usr/bin/env python
-# -*- coding: utf-8 -*-
 """Very basic hello-world example.
 """
 import time
-from pyxcp.cmdline import ArgumentParser
 
 import matplotlib.pyplot as plt
-import numpy as np
 import seaborn as sns
 
+from pyxcp.cmdline import ArgumentParser
+from pyxcp.transport import FrameRecorderPolicy
+
+
 sns.set()
 
 
@@ -16,13 +17,14 @@ ADDR = 0x4000
 LENGTH = 0x1000
 ITERATIONS = 100
 
-ap = ArgumentParser(description="pyXCP hello world.")
+recorder_policy = FrameRecorderPolicy()  # Create frame recorder.
+ap = ArgumentParser(description="pyXCP hello world.", policy=recorder_policy)
 with ap.run() as x:
     xs = []
     ys = []
     x.connect()
     for ctoSize in range(8, 64 + 4, 4):
-        print("CTO-Size: {}".format(ctoSize))
+        print(f"CTO-Size: {ctoSize}")
         xs.append(ctoSize)
         start = time.perf_counter()
         for _ in range(ITERATIONS):
@@ -30,7 +32,7 @@ with ap.run() as x:
             data = x.fetch(LENGTH, ctoSize)
         et = time.perf_counter() - start
         ys.append(et)
-        print("CTO size: {:-3} -- elapsed time {:-3.04}".format(ctoSize, et))
+        print(f"CTO size: {ctoSize:-3} -- elapsed time {et:-3.04}")
     x.disconnect()
     plt.plot(xs, ys)
     plt.show()
