@@ -1,5 +1,4 @@
 #!/usr/bin/env python
-# -*- coding: utf-8 -*-
 """Lowlevel API reflecting available XCP services
 
 .. note:: For technical reasons the API is split into two parts;
@@ -7,4 +6,4 @@
 
 .. [1] XCP Specification, Part 2 - Protocol Layer Specification
 """
-from .master import Master
+from .master import Master  # noqa: F401
