@@ -1,8 +1,6 @@
 #!/usr/bin/env python
-# -*- coding: utf-8 -*-
 """Scan for available IDs.
 """
-from pprint import pprint
 
 from pyxcp.cmdline import ArgumentParser
 
@@ -12,10 +10,8 @@ def main():
     with ap.run() as x:
         x.connect()
         result = x.id_scanner()
-        print("\n")
-        print("Implemented IDs".center(80))
-        print("=" * 80)
-        pprint(result)
+        for key, value in result.items():
+            print(f"{key}: {value}", end="\n\n")
         x.disconnect()
 
 
