@@ -1,12 +1,24 @@
 #!/usr/bin/env python
-# -*- coding: utf-8 -*-
 import struct
 import time
 from collections import deque
+from unittest import mock
+
 from pyxcp import types
 from pyxcp.master import Master
-from pyxcp.transport.can import CanInterfaceBase
-from unittest import mock
+
+
+def create_config():
+    # Exception: XCPonEth - Failed to resolve address <MagicMock name='mock.transport.eth.host' id='2414047113872'>:<MagicMock name='mock.transport.eth.port' id='2414047478992'>
+    config = mock.MagicMock()
+    config.general.return_value = mock.MagicMock()
+    config.transport.return_value = mock.MagicMock()
+    config.transport.eth.return_value = mock.MagicMock()
+    config.transport.eth.host = "localhost"
+    config.transport.eth.port = 5555
+    config.transport.eth.bind_to_address = ""
+    config.transport.eth.bind_to_port = 0
+    return config
 
 
 class MockSocket:
@@ -48,7 +60,7 @@ class MockSocket:
         pass
 
 
-class MockCanInterface(CanInterfaceBase):
+class MockCanInterface:  # CanInterfaceBase
     def __init__(self):
         self.data = deque()
         self.receive_callback = None
@@ -87,18 +99,17 @@ class MockCanInterface(CanInterfaceBase):
     def read(self):
         pass
 
-    def getTimestampResolution(self):
+    def get_timestamp_resolution(self):
         pass
 
 
 class TestMaster:
-
     DefaultConnectCmd = bytes([0x02, 0x00, 0x00, 0x00, 0xFF, 0x00])
     DefaultConnectResponse = "FF 3D C0 FF DC 05 01 01"
 
     @mock.patch("pyxcp.transport.eth")
     def testConnect(self, eth):
-        with Master("eth") as xm:
+        with Master("eth", config=create_config()) as xm:
             xm.transport = eth()
             xm.transport.request.return_value = bytes([0x1D, 0xC0, 0xFF, 0xDC, 0x05, 0x01, 0x01])
 
@@ -121,15 +132,15 @@ class TestMaster:
 
     @mock.patch("pyxcp.transport.eth")
     def testDisconnect(self, eth):
-        with Master("eth") as xm:
+        with Master("eth", config=create_config()) as xm:
             xm.transport = eth()
-            xm.transport.request.return_value = bytes([])
+            xm.transport.request_optional_response.return_value = bytes([])
             res = xm.disconnect()
         assert res == b""
 
     @mock.patch("pyxcp.transport.eth")
     def testGetStatus(self, eth):
-        with Master("eth") as xm:
+        with Master("eth", config=create_config()) as xm:
             xm.transport = eth()
             xm.transport.request.return_value = bytes([0x1D, 0xC0, 0xFF, 0xDC, 0x05, 0x01, 0x01])
 
@@ -152,7 +163,7 @@ class TestMaster:
 
     @mock.patch("pyxcp.transport.eth")
     def testSync(self, eth):
-        with Master("eth") as xm:
+        with Master("eth", config=create_config()) as xm:
             xm.transport = eth()
             xm.transport.request.return_value = bytes([0x00])
             res = xm.synch()
@@ -160,7 +171,7 @@ class TestMaster:
 
     @mock.patch("pyxcp.transport.eth")
     def testGetCommModeInfo(self, eth):
-        with Master("eth") as xm:
+        with Master("eth", config=create_config()) as xm:
             xm.transport = eth()
             xm.transport.request.return_value = bytes([0x1D, 0xC0, 0xFF, 0xDC, 0x05, 0x01, 0x01])
 
@@ -179,7 +190,7 @@ class TestMaster:
 
     @mock.patch("pyxcp.transport.eth")
     def testGetId(self, eth):
-        with Master("eth") as xm:
+        with Master("eth", config=create_config()) as xm:
             xm.transport = eth()
             xm.transport.MAX_DATAGRAM_SIZE = 512
             xm.transport.request.return_value = bytes([0x1D, 0xC0, 0xFF, 0xDC, 0x05, 0x01, 0x01])
@@ -204,7 +215,7 @@ class TestMaster:
         mock_socket.return_value.recv.side_effect = ms.recv
         mock_selector.return_value.select.side_effect = ms.select
 
-        with Master("eth", config={"HOST": "localhost", "LOGLEVEL": "DEBUG"}) as xm:
+        with Master("eth", config=create_config()) as xm:
             ms.push_packet(self.DefaultConnectResponse)
 
             res = xm.connect()
@@ -248,7 +259,7 @@ class TestMaster:
         mock_socket.return_value.recv.side_effect = ms.recv
         mock_selector.return_value.select.side_effect = ms.select
 
-        with Master("eth", config={"HOST": "localhost", "LOGLEVEL": "DEBUG"}) as xm:
+        with Master("eth", config=create_config()) as xm:
             ms.push_packet(self.DefaultConnectResponse)
 
             res = xm.connect()
@@ -271,7 +282,7 @@ class TestMaster:
         mock_socket.return_value.recv.side_effect = ms.recv
         mock_selector.return_value.select.side_effect = ms.select
 
-        with Master("eth", config={"HOST": "localhost", "LOGLEVEL": "DEBUG"}) as xm:
+        with Master("eth", config=create_config()) as xm:
             ms.push_packet(self.DefaultConnectResponse)
 
             res = xm.connect()
@@ -303,7 +314,7 @@ class TestMaster:
         mock_socket.return_value.recv.side_effect = ms.recv
         mock_selector.return_value.select.side_effect = ms.select
 
-        with Master("eth", config={"HOST": "localhost", "LOGLEVEL": "DEBUG"}) as xm:
+        with Master("eth", config=create_config()) as xm:
             ms.push_packet(self.DefaultConnectResponse)
 
             res = xm.connect()
@@ -326,7 +337,7 @@ class TestMaster:
         mock_socket.return_value.recv.side_effect = ms.recv
         mock_selector.return_value.select.side_effect = ms.select
 
-        with Master("eth", config={"HOST": "localhost", "LOGLEVEL": "DEBUG"}) as xm:
+        with Master("eth", config=create_config()) as xm:
             ms.push_packet(self.DefaultConnectResponse)
 
             res = xm.connect()
@@ -354,7 +365,7 @@ class TestMaster:
         mock_socket.return_value.recv.side_effect = ms.recv
         mock_selector.return_value.select.side_effect = ms.select
 
-        with Master("eth", config={"HOST": "localhost", "LOGLEVEL": "DEBUG"}) as xm:
+        with Master("eth", config=create_config()) as xm:
             ms.push_packet(self.DefaultConnectResponse)
 
             res = xm.connect()
@@ -417,7 +428,7 @@ class TestMaster:
         mock_socket.return_value.recv.side_effect = ms.recv
         mock_selector.return_value.select.side_effect = ms.select
 
-        with Master("eth", config={"HOST": "localhost", "LOGLEVEL": "DEBUG"}) as xm:
+        with Master("eth", config=create_config()) as xm:
             ms.push_packet(self.DefaultConnectResponse)
 
             res = xm.connect()
@@ -440,7 +451,7 @@ class TestMaster:
         mock_socket.return_value.recv.side_effect = ms.recv
         mock_selector.return_value.select.side_effect = ms.select
 
-        with Master("eth", config={"HOST": "localhost", "LOGLEVEL": "DEBUG"}) as xm:
+        with Master("eth", config=create_config()) as xm:
             ms.push_packet(self.DefaultConnectResponse)
 
             res = xm.connect()
@@ -464,7 +475,7 @@ class TestMaster:
         mock_socket.return_value.recv.side_effect = ms.recv
         mock_selector.return_value.select.side_effect = ms.select
 
-        with Master("eth", config={"HOST": "localhost", "LOGLEVEL": "DEBUG"}) as xm:
+        with Master("eth", config=create_config()) as xm:
             ms.push_packet(self.DefaultConnectResponse)
 
             res = xm.connect()
@@ -490,7 +501,7 @@ class TestMaster:
         mock_socket.return_value.recv.side_effect = ms.recv
         mock_selector.return_value.select.side_effect = ms.select
 
-        with Master("eth", config={"HOST": "localhost", "LOGLEVEL": "DEBUG"}) as xm:
+        with Master("eth", config=create_config()) as xm:
             ms.push_packet(self.DefaultConnectResponse)
 
             res = xm.connect()
@@ -530,7 +541,7 @@ class TestMaster:
         mock_socket.return_value.recv.side_effect = ms.recv
         mock_selector.return_value.select.side_effect = ms.select
 
-        with Master("eth", config={"HOST": "localhost", "LOGLEVEL": "DEBUG"}) as xm:
+        with Master("eth", config=create_config()) as xm:
             ms.push_packet(self.DefaultConnectResponse)
 
             res = xm.connect()
@@ -569,7 +580,7 @@ class TestMaster:
         mock_socket.return_value.recv.side_effect = ms.recv
         mock_selector.return_value.select.side_effect = ms.select
 
-        with Master("eth", config={"HOST": "localhost", "LOGLEVEL": "DEBUG"}) as xm:
+        with Master("eth", config=create_config()) as xm:
             ms.push_packet(self.DefaultConnectResponse)
 
             res = xm.connect()
@@ -609,7 +620,7 @@ class TestMaster:
         mock_socket.return_value.recv.side_effect = ms.recv
         mock_selector.return_value.select.side_effect = ms.select
 
-        with Master("eth", config={"HOST": "localhost", "LOGLEVEL": "DEBUG"}) as xm:
+        with Master("eth", config=create_config()) as xm:
             ms.push_packet(self.DefaultConnectResponse)
 
             res = xm.connect()
@@ -650,7 +661,7 @@ class TestMaster:
         mock_socket.return_value.recv.side_effect = ms.recv
         mock_selector.return_value.select.side_effect = ms.select
 
-        with Master("eth", config={"HOST": "localhost", "LOGLEVEL": "DEBUG"}) as xm:
+        with Master("eth", config=create_config()) as xm:
             ms.push_packet(self.DefaultConnectResponse)
 
             res = xm.connect()
@@ -674,7 +685,7 @@ class TestMaster:
         mock_socket.return_value.recv.side_effect = ms.recv
         mock_selector.return_value.select.side_effect = ms.select
 
-        with Master("eth", config={"HOST": "localhost", "LOGLEVEL": "DEBUG"}) as xm:
+        with Master("eth", config=create_config()) as xm:
             ms.push_packet(self.DefaultConnectResponse)
 
             res = xm.connect()
@@ -698,7 +709,7 @@ class TestMaster:
         mock_socket.return_value.recv.side_effect = ms.recv
         mock_selector.return_value.select.side_effect = ms.select
 
-        with Master("eth", config={"HOST": "localhost", "LOGLEVEL": "DEBUG"}) as xm:
+        with Master("eth", config=create_config()) as xm:
             ms.push_packet(self.DefaultConnectResponse)
 
             res = xm.connect()
@@ -724,7 +735,7 @@ class TestMaster:
         mock_socket.return_value.recv.side_effect = ms.recv
         mock_selector.return_value.select.side_effect = ms.select
 
-        with Master("eth", config={"HOST": "localhost", "LOGLEVEL": "DEBUG"}) as xm:
+        with Master("eth", config=create_config()) as xm:
             ms.push_packet(self.DefaultConnectResponse)
 
             res = xm.connect()
@@ -748,7 +759,7 @@ class TestMaster:
             "CAN_USE_DEFAULT_LISTENER": False,
         }
         with Master("can", config=conf) as xm:
-            mock_caninterface = xm.transport.canInterface
+            mock_caninterface = xm.transport.can_interface
             mock_caninterface.push_packet(self.DefaultConnectResponse)
             xm.connect()
 
@@ -817,7 +828,7 @@ class TestMaster:
             "CAN_USE_DEFAULT_LISTENER": False,
         }
         with Master("can", config=conf) as xm:
-            mock_caninterface = xm.transport.canInterface
+            mock_caninterface = xm.transport.can_interface
             mock_caninterface.push_packet(self.DefaultConnectResponse)
             xm.connect()
 
