@@ -1,13 +1,30 @@
-import pyxcp.transport.base as tr
+from unittest import mock
 
 import pytest
 
+import pyxcp.transport.base as tr
+
+
+def create_config():
+    # Exception: XCPonEth - Failed to resolve address <MagicMock name='mock.transport.eth.host' id='2414047113872'>:<MagicMock name='mock.transport.eth.port' id='2414047478992'>
+    config = mock.MagicMock()
+    config.general.return_value = mock.MagicMock()
+    config.transport.return_value = mock.MagicMock()
+    config.transport.eth.return_value = mock.MagicMock()
+    config.transport.eth.host = "localhost"
+    config.transport.eth.port = 5555
+    config.transport.eth.bind_to_address = ""
+    config.transport.eth.bind_to_port = 0
+    config.transport.create_daq_timestamps = False
+    return config
+
 
 def test_factory_works():
-    assert isinstance(tr.createTransport("eth"), tr.BaseTransport)
-    assert isinstance(tr.createTransport("sxi"), tr.BaseTransport)
+    config = create_config()
+    assert isinstance(tr.create_transport("eth", config=config), tr.BaseTransport)
+    assert isinstance(tr.create_transport("sxi", config=config), tr.BaseTransport)
     assert isinstance(
-        tr.createTransport(
+        tr.create_transport(
             "can",
             config={
                 "CAN_ID_MASTER": 1,
@@ -20,10 +37,10 @@ def test_factory_works():
 
 
 def test_factory_works_case_insensitive():
-    assert isinstance(tr.createTransport("ETH"), tr.BaseTransport)
-    assert isinstance(tr.createTransport("SXI"), tr.BaseTransport)
+    assert isinstance(tr.create_transport("ETH"), tr.BaseTransport)
+    assert isinstance(tr.create_transport("SXI"), tr.BaseTransport)
     assert isinstance(
-        tr.createTransport(
+        tr.create_transport(
             "CAN",
             config={
                 "CAN_ID_MASTER": 1,
@@ -37,11 +54,11 @@ def test_factory_works_case_insensitive():
 
 def test_factory_invalid_transport_name_raises():
     with pytest.raises(ValueError):
-        tr.createTransport("xCp")
+        tr.create_transport("xCp")
 
 
 def test_transport_names():
-    transports = tr.availableTransports()
+    transports = tr.available_transports()
 
     assert "can" in transports
     assert "eth" in transports
@@ -49,7 +66,7 @@ def test_transport_names():
 
 
 def test_transport_names_are_lower_case_only():
-    transports = tr.availableTransports()
+    transports = tr.available_transports()
 
     assert "CAN" not in transports
     assert "ETH" not in transports
@@ -57,7 +74,7 @@ def test_transport_names_are_lower_case_only():
 
 
 def test_transport_classes():
-    transports = tr.availableTransports()
+    transports = tr.available_transports()
 
     assert issubclass(transports.get("can"), tr.BaseTransport)
     assert issubclass(transports.get("eth"), tr.BaseTransport)
