@@ -1,60 +1,78 @@
 #!/usr/bin/env python
-# -*- coding: utf-8 -*-
 import selectors
 import socket
 import struct
 import threading
 from collections import deque
+from typing import Optional
+
 from pyxcp.transport.base import BaseTransport
-from time import perf_counter
-from time import sleep
-from time import time
+from pyxcp.utils import short_sleep
+
 
 DEFAULT_XCP_PORT = 5555
 RECV_SIZE = 8196
 
 
+def socket_to_str(sock: socket.socket) -> str:
+    peer = sock.getpeername()
+    local = sock.getsockname()
+    AF = {
+        socket.AF_INET: "AF_INET",
+        socket.AF_INET6: "AF_INET6",
+    }
+    TYPE = {
+        socket.SOCK_DGRAM: "SOCK_DGRAM",
+        socket.SOCK_STREAM: "SOCK_STREAM",
+    }
+    family = AF.get(sock.family, "OTHER")
+    typ = TYPE.get(sock.type, "UNKNOWN")
+    res = f"XCPonEth - Connected to: {peer[0]}:{peer[1]}  local address: {local[0]}:{local[1]} [{family}][{typ}]"
+    return res
+
+
 class Eth(BaseTransport):
     """"""
 
-    PARAMETER_MAP = {
-        #                  Type    Req'd   Default
-        "HOST": (str, False, "localhost"),
-        "PORT": (int, False, 5555),
-        "PROTOCOL": (str, False, "TCP"),
-        "IPV6": (bool, False, False),
-        "TCP_NODELAY": (bool, False, False),
-    }
-
     MAX_DATAGRAM_SIZE = 512
     HEADER = struct.Struct("<HH")
     HEADER_SIZE = HEADER.size
 
-    def __init__(self, config=None):
-        super(Eth, self).__init__(config)
-        self.loadConfig(config)
-        self.host = self.config.get("HOST")
-        self.port = self.config.get("PORT")
-        self.protocol = self.config.get("PROTOCOL")
-        self.ipv6 = self.config.get("IPV6")
-        self.use_tcp_no_delay = self.config.get("TCP_NODELAY")
-
+    def __init__(self, config=None, policy=None, transport_layer_interface: Optional[socket.socket] = None) -> None:
+        super().__init__(config, policy, transport_layer_interface)
+        self.load_config(config)
+        self.host: str = self.config.host
+        self.port: int = self.config.port
+        self.protocol: int = self.config.protocol
+        self.ipv6: bool = self.config.ipv6
+        self.use_tcp_no_delay: bool = self.config.tcp_nodelay
+        address_to_bind: str = self.config.bind_to_address
+        bind_to_port: int = self.config.bind_to_port
+        self._local_address = (address_to_bind, bind_to_port) if address_to_bind else None
         if self.ipv6 and not socket.has_ipv6:
-            raise RuntimeError("IPv6 not supported by your platform.")
+            msg = "XCPonEth - IPv6 not supported by your platform."
+            self.logger.critical(msg)
+            raise RuntimeError(msg)
         else:
             address_family = socket.AF_INET6 if self.ipv6 else socket.AF_INET
         proto = socket.SOCK_STREAM if self.protocol == "TCP" else socket.SOCK_DGRAM
         if self.host.lower() == "localhost":
             self.host = "::1" if self.ipv6 else "localhost"
-        addrinfo = socket.getaddrinfo(self.host, self.port, address_family, proto)
-        (
-            self.address_family,
-            self.socktype,
-            self.proto,
-            self.canonname,
-            self.sockaddr,
-        ) = addrinfo[0]
-        self.status = 0
+
+        try:
+            addrinfo = socket.getaddrinfo(self.host, self.port, address_family, proto)
+            (
+                self.address_family,
+                self.socktype,
+                self.proto,
+                self.canonname,
+                self.sockaddr,
+            ) = addrinfo[0]
+        except BaseException as ex:  # noqa: B036
+            msg = f"XCPonEth - Failed to resolve address {self.host}:{self.port}"
+            self.logger.critical(msg)
+            raise Exception(msg) from ex
+        self.status: int = 0
         self.sock = socket.socket(self.address_family, self.socktype, self.proto)
         self.selector = selectors.DefaultSelector()
         self.selector.register(self.sock, selectors.EVENT_READ)
@@ -65,7 +83,13 @@ class Eth(BaseTransport):
         if hasattr(socket, "SO_REUSEPORT"):
             self.sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEPORT, 1)
         self.sock.settimeout(0.5)
-
+        if self._local_address:
+            try:
+                self.sock.bind(self._local_address)
+            except BaseException as ex:  # noqa: B036
+                msg = f"XCPonEth - Failed to bind socket to given address {self._local_address}"
+                self.logger.critical(msg)
+                raise Exception(msg) from ex
         self._packet_listener = threading.Thread(
             target=self._packet_listen,
             args=(),
@@ -73,44 +97,40 @@ class Eth(BaseTransport):
         )
         self._packets = deque()
 
-    def connect(self):
+    def connect(self) -> None:
         if self.status == 0:
             self.sock.connect(self.sockaddr)
-            self.startListener()
+            self.logger.info(socket_to_str(self.sock))
+            self.start_listener()
             self.status = 1  # connected
 
-    def startListener(self):
+    def start_listener(self) -> None:
+        super().start_listener()
+        if self._packet_listener.is_alive():
+            self._packet_listener.join()
+        self._packet_listener = threading.Thread(target=self._packet_listen)
         self._packet_listener.start()
-        self.listener.start()
 
-    def close(self):
+    def close(self) -> None:
         """Close the transport-layer connection and event-loop."""
-        self.finishListener()
+        self.finish_listener()
         if self.listener.is_alive():
             self.listener.join()
         if self._packet_listener.is_alive():
             self._packet_listener.join()
-        self.closeConnection()
+        self.close_connection()
 
-    def _packet_listen(self):
-        use_tcp = self.use_tcp
+    def _packet_listen(self) -> None:
+        use_tcp: bool = self.use_tcp
         EVENT_READ = selectors.EVENT_READ
-
         close_event_set = self.closeEvent.is_set
         socket_fileno = self.sock.fileno
         select = self.selector.select
-
-        high_resolution_time = self.perf_counter_origin < 0
-        timestamp_origin = self.timestamp_origin
-        perf_counter_origin = self.perf_counter_origin
-
         _packets = self._packets
-
         if use_tcp:
             sock_recv = self.sock.recv
         else:
             sock_recv = self.sock.recvfrom
-
         while True:
             try:
                 if close_event_set() or socket_fileno() == -1:
@@ -118,11 +138,7 @@ class Eth(BaseTransport):
                 sel = select(0.02)
                 for _, events in sel:
                     if events & EVENT_READ:
-                        if high_resolution_time:
-                            recv_timestamp = time()
-                        else:
-                            recv_timestamp = timestamp_origin + perf_counter() - perf_counter_origin
-
+                        recv_timestamp = self.timestamp.value
                         if use_tcp:
                             response = sock_recv(RECV_SIZE)
                             if not response:
@@ -139,41 +155,33 @@ class Eth(BaseTransport):
                                 break
                             else:
                                 _packets.append((response, recv_timestamp))
-            except BaseException:
+            except BaseException:  # noqa: B036
                 self.status = 0  # disconnected
                 break
 
-    def listen(self):
+    def listen(self) -> None:
         HEADER_UNPACK_FROM = self.HEADER.unpack_from
         HEADER_SIZE = self.HEADER_SIZE
-        processResponse = self.processResponse
+        process_response = self.process_response
         popleft = self._packets.popleft
-
         close_event_set = self.closeEvent.is_set
         socket_fileno = self.sock.fileno
-
         _packets = self._packets
-        length, counter = None, None
-
-        data = bytearray(b"")
-
+        length: Optional[int] = None
+        counter: int = 0
+        data: bytearray = bytearray(b"")
         while True:
             if close_event_set() or socket_fileno() == -1:
                 return
-
-            count = len(_packets)
-
+            count: int = len(_packets)
             if not count:
-                sleep(0.002)
+                short_sleep()
                 continue
-
             for _ in range(count):
                 bts, timestamp = popleft()
-
                 data += bts
-                current_size = len(data)
-                current_position = 0
-
+                current_size: int = len(data)
+                current_position: int = 0
                 while True:
                     if length is None:
                         if current_size >= HEADER_SIZE:
@@ -185,33 +193,21 @@ class Eth(BaseTransport):
                             break
                     else:
                         if current_size >= length:
-                            # response = memoryview(data[current_position : current_position + length])
                             response = data[current_position : current_position + length]
-                            processResponse(response, length, counter, timestamp)
-
+                            process_response(response, length, counter, timestamp)
                             current_size -= length
                             current_position += length
-
                             length = None
-
                         else:
-
                             data = data[current_position:]
                             break
 
-    def send(self, frame):
-        if self.perf_counter_origin < 0:
-            self.pre_send_timestamp = time()
-            self.sock.send(frame)
-            self.post_send_timestamp = time()
-        else:
-            pre_send_timestamp = perf_counter()
-            self.sock.send(frame)
-            post_send_timestamp = perf_counter()
-            self.pre_send_timestamp = self.timestamp_origin + pre_send_timestamp - self.perf_counter_origin
-            self.post_send_timestamp = self.timestamp_origin + post_send_timestamp - self.perf_counter_origin
+    def send(self, frame) -> None:
+        self.pre_send_timestamp = self.timestamp.value
+        self.sock.send(frame)
+        self.post_send_timestamp = self.timestamp.value
 
-    def closeConnection(self):
+    def close_connection(self) -> None:
         if not self.invalidSocket:
             # Seems to be problematic /w IPv6
             # if self.status == 1:
@@ -219,5 +215,5 @@ class Eth(BaseTransport):
             self.sock.close()
 
     @property
-    def invalidSocket(self):
+    def invalidSocket(self) -> bool:
         return not hasattr(self, "sock") or self.sock.fileno() == -1
